#ifndef _LGSITERATIV_H_
#define _LGSITERATIV_H_

// Hilfsprogramme 
#include <iostream>
using namespace std;
#include <cstdlib>

void CGDiagCSR(int* column_indices,double* entries,int* row_offsets,
	       double* x,double* b,int n,int itmax, double epsi,int *it)
// CG-Verfahren zum Loesen von Ax=b
// A\in R^{n,n} als CSR-Matrix gespeichert
// Diagonale Vorkonditionierung
// eps.. relative Genauigkeit
// itmax.. max Iterationszahl
// it.. benoetigte Iterationen (Ausgabe)
// x.. Start/Loesungsvektor (I/O)
{
  double* diag;
  double* s;
  double* r;
  double* w;
  double sas,abbr,rw,rwv,alpha,beta;
  int i;
  diag=(double*)malloc(n*sizeof(double));
  r=(double*)malloc(n*sizeof(double));
  w=(double*)malloc(n*sizeof(double));
  s=(double*)malloc(n*sizeof(double));
  GenDiagCSR(column_indices,entries,row_offsets,diag,n);
  i=0;
  MatVecMulCSR(column_indices,entries,row_offsets,x,s,n);// s=A*x
  vdminus(n,s,1,b,1,r,1); // r=s-b
  vddiv(n,r,1,diag,1,w,1); // w=D^(-1)r
  dscapr(&rw,n,r,w); // rw=r'*w;
  beta=0;
  rwv=rw;
  abbr=epsi*epsi*rwv;
  cout <<"Iterationen  <r,w>"<<endl;
  while (i<itmax && abbr<rwv)
    {
      i++;
      vdaxpy(n,s,beta,w,s);
      MatVecMulCSR(column_indices,entries,row_offsets,s,w,n); // w=A*s;
      dscapr(&sas,n,s,w); // sas=s'*A*s;
      alpha=-rw/sas;
      vdaxpy(n,s,alpha,x,x);
      vdaxpy(n,w,alpha,r,r);
      vddiv(n,r,1,diag,1,w,1); // w=D^(-1)r
      dscapr(&rw,n,r,w); // rw=r'*w;
      beta=rw/rwv;
      rwv=rw;
      cout<< i <<"    "<< rw <<endl;
    }
  *it=i;
  free(diag); 
  free(r);
  free(w);
  free(s);
}

void JacobiCSR(int* column_indices,double* entries,int* row_offsets,
	       double* x,double* b,int n,int itmax, double eps,int *it)
// Jacobi-Verfahren zum Loesen von Ax=b
// A\in R^{n,n} als CSR-Matrix gespeichert
// eps.. relative Genauigkeit
// itmax.. max Iterationszahl
// it.. benoetigte Iterationen (Ausgabe)
// x.. Start/Loesungsvektor (I/O)
{
  double* diag;
  double* w;
  double* r;
  double abbr,rw;
  int i;
  diag=(double*)malloc(n*sizeof(double));
  r=(double*)malloc(n*sizeof(double));
  w=(double*)malloc(n*sizeof(double));
  GenDiagCSR(column_indices,entries,row_offsets,diag,n);
  i=0;
  MatVecMulCSR(column_indices,entries,row_offsets,x,r,n);
  vdminus(n,r,1,b,1,r,1);  // r=A*x-b
  vddiv(n,r,1,diag,1,w,1);  // w=D^{-1}r
  dscapr(&rw,n,r,w);
  abbr=eps*eps*rw;
  cout <<"Iterationen  <r,w>"<<endl;
  while (i<itmax && abbr<rw)
    {
      i++;
      vdminus(n,x,1,w,1,x,1);
      MatVecMulCSR(column_indices,entries,row_offsets,x,r,n);
      vdminus(n,r,1,b,1,r,1);
      vddiv(n,r,1,diag,1,w,1);
      dscapr(&rw,n,r,w);
      cout<< i <<"    "<< rw <<endl;
    }
  *it=i;
  free(diag); 
  free(r);
  free(w);
}


#endif
